package com.sundata.internalevaluation.configuration.service;

import com.sundata.common.util.JsonUtil;
import com.sundata.internalevaluation.configuration.mapper.IndexListConfigMapper;
import com.sundata.internalevaluation.configuration.model.IndexListModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class IndexListConfigService {

    @Autowired
    IndexListConfigMapper indexListConfigMapper;

    /**
     * @author: 张艳杰
     * @methodsName: getDataList
     * @description: 指标清单配置初始化
     * @param:  IndexListModel model
     * @return: List<IndexListModel>
     * @throws:
     */
    public List<IndexListModel> getDataList(IndexListModel model) {
        return indexListConfigMapper.getDataList(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: createData
     * @description: 指标清单配置新增
     * @param:  IndexListModel model
     * @return:
     * @throws:
     */
    public void createData(IndexListModel model) {
        if (null != model && null != model.getIndexNos() && model.getIndexNos().size()> 0) {
            for ( String s : model.getIndexNos()) {
                model.setIndexNo(s);
                indexListConfigMapper.createAssociation(model);
            }
        }
        indexListConfigMapper.createData(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: deleteRows
     * @description: 指标清单配置删除
     * @param:  IndexListModel model
     * @return:
     * @throws:
     */
    public void deleteRows(IndexListModel model) {
        indexListConfigMapper.deleteRows(model.getIndexListNo());
        indexListConfigMapper.deleteAssociation(model.getIndexListNo());
    }

    /**
     * @author: 张艳杰
     * @methodsName: updateExistData
     * @description: 指标清单配置修改
     * @param:  IndexListModel model
     * @return:
     * @throws:
     */
    public void updateExistData(IndexListModel model) {
        indexListConfigMapper.updateExistData(model);
        indexListConfigMapper.deleteAssociation(model.getIndexListNo());
        if (null != model && null != model.getIndexNos() && model.getIndexNos().size()> 0) {
            for ( String s : model.getIndexNos()) {
                model.setIndexNo(s);
                indexListConfigMapper.createAssociation(model);
            }
        }
    }

    /**
     * @author: 张艳杰
     * @methodsName: selectDetailData
     * @description: 指标清单配置查询
     * @param:  IndexListModel model
     * @return:
     * @throws:
     */
    public IndexListModel selectDetailData(IndexListModel model) {
        IndexListModel selectModel = indexListConfigMapper.selectDetailData(model);
        List<String> indexList = new ArrayList<>();
        List<IndexListModel> associationList = indexListConfigMapper.selectAssociation(selectModel);
        if (null != associationList && associationList.size()>0) {
            for (IndexListModel m : associationList) {
                indexList.add(m.getIndexNo());
            }
        }
        selectModel.setIndexNos(indexList);
        return selectModel;
    }
}
