package com.sundata.internalevaluation.script.pool;

import groovy.text.GStringTemplateEngine;
import groovy.text.Template;
import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.PooledObjectFactory;
import org.apache.commons.pool2.impl.PooledSoftReference;
import com.sundata.internalevaluation.script.model.TemplateModel;

import java.lang.ref.SoftReference;

// 创建一个实现 PooledObjectFactory 的类，用于生成和管理 TemplateModel 对象
public class GroovyTemplateFactory implements PooledObjectFactory<Template> {
    private final GStringTemplateEngine engine = new GStringTemplateEngine();
    private final TemplateModel template;

    public GroovyTemplateFactory(TemplateModel template) {
        this.template = template;
    }

    @Override
    public PooledObject<Template> makeObject() throws Exception {
        return new PooledSoftReference<>(new SoftReference<>(engine.createTemplate(template.getTemplateStr())));
    }

    @Override
    public void destroyObject(PooledObject<Template> p) {
//        p.deallocate()
    }

    @Override
    public boolean validateObject(PooledObject<Template> p) {
        // 简单地验证对象是否不为 null
        return p.getObject() != null;
    }

    @Override
    public void activateObject(PooledObject<Template> p) throws Exception {
        // 可以在此处重新初始化模板对象如果需要
    }

    @Override
    public void passivateObject(PooledObject<Template> p) throws Exception {
        // 可以在此处清理模板对象的状态如果需要
    }
}
